/*
   Dot version 1.0  Copyright (C) 1999 Frank Reglin

   Dot is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Dot is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Dot; see the file COPYING.
*/
/* dot screen saver
   windows 32 version
   Frank Reglin 1999

   Idea by Bettina Reglin

   see ct 25/98
*/

#include <windows.h>
#include <scrnsave.h>
#include <stdio.h>

static long x0, y0, xmax, ymax;
static long xpos = -1;   /* dot position */
static long ypos = -1;   /* dot position */

#define DOTSIZE 4        /* DOT = DOTSIZE*DOTSIZE pixels */

#define BLACK RGB(0,0,0)
#define WHITE RGB(255,255,255)

#define TIME_PERIOD 100

#define FILENAME "c:\\zzsave.dot"



void setdot( HDC hdc, long x, long y, long color )
/***************************************************************/
{
int xx, yy, xabs, yabs;

   for( xx = 0; xx < DOTSIZE; xx++ )
   {
      xabs = x0 + x * DOTSIZE + xx;
      for( yy = 0; yy < DOTSIZE; yy++ )
      {
         yabs = y0 + y * DOTSIZE + yy;
         SetPixel( hdc, xabs, yabs, color );
      }
   }
}



BOOL WINAPI RegisterDialogClasses(HANDLE hInst)
/***************************************************************/
{
   return TRUE;
}


BOOL WINAPI ScreenSaverConfigureDialog(HWND hDlg, UINT message,
                                    WPARAM wParam, LPARAM lParam)
/***************************************************************/
{
   switch(message) 
   {
   case WM_INITDIALOG:
   return TRUE;

      case WM_COMMAND:
         EndDialog( hDlg,TRUE);
         return TRUE;

   }
   return FALSE;
}


LONG WINAPI ScreenSaverProc(HWND hWnd, UINT message,
                                    WPARAM wParam, LPARAM lParam)
/***************************************************************/
{
HDC hDC;
FILE * f;
int n;
static RECT	screen;
static HBRUSH black;

   switch( message )
   {
      case WM_CREATE:
         SetTimer( hWnd, 1 /* timer id */, TIME_PERIOD, NULL );
         black = CreateSolidBrush( BLACK );
         GetClientRect( hWnd, &screen );

         /* origin, should be 0,0 : */
         x0 = screen.left;
         y0 = screen.top;

         /* max coordinates, relative to the origin: */
         xmax = (screen.right - x0) / DOTSIZE - 1;
         ymax = (screen.bottom - y0) / DOTSIZE;
         if( !(ymax & 1) )
            ymax--;                       /* has to be odd ! */
         break;

      case WM_TIMER:
         hDC = GetDC( hWnd );

         /* Paint the screen black */
//         FillRect( hDC, &screen, black );

         /* If the dot's position is unknown, initialize it */
         if( xpos < 0 )
         {
            /* try to get it from the file: */
            f = fopen( FILENAME, "r" );
            if( f )
            {
               n = fscanf( f, "%ld %ld\n", &xpos, &ypos );
               fclose( f );
               if( n == 2 )
               {
                  if( xpos > xmax )
                     xpos = xmax;
                  else if( xpos < 0 )
                     xpos = 0;
                  if( ypos > ymax )
                     ypos = ymax;
                  else if( ypos < 0 )
                     ypos = 0;
               }
               else
               {
                  xpos = 0;
                  ypos = 0;
               }
            }
            else
            {
               xpos = 0;
               ypos = 0;
            }
         }

         /* Clear the dot at the old position: */
         setdot( hDC, xpos, ypos, BLACK );

         /* Find the new position of the dot */
         if( ypos & 1L )
         {              /* odd: */
            xpos--;
            if( xpos < 0 )
            {
               xpos = 0;
               ypos++;
               if( ypos > ymax )
                  ypos = 0;
            }
         }
         else
         {              /* even: */
            xpos++;
            if( xpos > xmax )
            {
               xpos = xmax;
               ypos++;
           }
         }

         /* Draw the dot at the new position: */
         setdot( hDC, xpos, ypos, WHITE );
   
         ReleaseDC( hWnd, hDC );
         break;

      case WM_DESTROY:	/* Beenden des BSS */
         KillTimer( hWnd, 1 /* timer id */ );
         DeleteObject( black );

         /* save actual coordinates to a file: */
         f = fopen( FILENAME, "w" );
         if( f )
         {
            fprintf( f, "%ld %ld\n", xpos, ypos );
            fclose( f );
         }

         break;
   }
   return DefScreenSaverProc( hWnd, message, wParam, lParam );
}

